/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.federation.service;

import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.opensaml.SAMLAttribute;
import org.opensaml.SAMLAttributeStatement;
import org.opensaml.SAMLException;
import org.opensaml.SAMLNameIdentifier;
import org.opensaml.SAMLSubject;

import org.jboss.security.util.Tools;

/**
 * Trust provides an implementation of the TrustService that can be managed via the jmx-console
 * 
 * @author Sohil Shah - sohil.shah@jboss.com - Oct 31, 2006
 *
 */
public class Trust implements TrustService 
{
    private static Trust singleton = null;
    
    /**
     * 
     */
    private Map secretTokens = new HashMap();
    private SecureRandom random = null;
    
    /**
     * 
     *
     */
    private Trust() 
    {
        super();
        try
        {
            this.random = SecureRandom.getInstance("SHA1PRNG");
            this.random.setSeed(SecureRandom.getSeed(8));
        }
        catch(Exception e)
        {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * Creates an instance of the Trust Service for this instance of the Federation Server
     * 
     * @return an instance of the Trust Service
     */
    public static Trust getInstance()
    {
        if(Trust.singleton == null)
        {
            Trust.singleton = new Trust();
        }
        return Trust.singleton;
    }

    /**
     * Generates a token secret understood by the Federation of partner applications only
     * 
     * @param tokenIdentifier an identifier that can associate a secret with the corresponding token
     * @return the secret
     */
    public String generateSecret(String tokenIdentifier) 
    {
        try
        {
            String secret = null;
            
            byte[] bytes = new byte[512];
            this.random.nextBytes(bytes);
            secret = Tools.encodeHex(bytes);
            this.secretTokens.put(tokenIdentifier,secret);
            
            //create a SAML envelope and return the secret in this envelope
            SAMLAttribute tokenSecret = new SAMLAttribute();
            tokenSecret.setName("secret");
            tokenSecret.setNamespace("jbosssso:secret");
            Collection values = new ArrayList();
            values.add(secret);
            tokenSecret.setValues(values);
            SAMLSubject tokenSubject = this.getSubject(tokenIdentifier);
            SAMLAttributeStatement secretStatement = new SAMLAttributeStatement();
            secretStatement.addAttribute(tokenSecret);
            secretStatement.setSubject(tokenSubject);
            
            return secretStatement.toString();
        }
        catch(Exception e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * Validates the secret associated with the token being processed by the Single Sign On components
     * 
     * @param tokenIdentifier an identifier that can associate a secret with the corresponding token
     * @param secret the secret associated with the token being processed by the Single Sign On components
     * @return a status of whether this secret is valid or not
     */
    public String validateSecret(String tokenIdentifier,String secret) 
    { 
        try
        {
            boolean isSecretValid = false;
            
            String storedSecret = (String)this.secretTokens.get(tokenIdentifier);
            if(storedSecret!=null && storedSecret.equals(secret))
            {
                isSecretValid = true;
            }
            
            //create a SAML envelope and return the secret in this envelope
            SAMLAttribute validStatus = new SAMLAttribute();
            validStatus.setName("valid");
            validStatus.setNamespace("jbosssso:secret");
            Collection values = new ArrayList();
            values.add(new Boolean(isSecretValid).toString());
            validStatus.setValues(values);
            SAMLSubject tokenSubject = this.getSubject(tokenIdentifier);
            SAMLAttributeStatement validStatement = new SAMLAttributeStatement();
            validStatement.addAttribute(validStatus);
            validStatement.setSubject(tokenSubject);
            
            return validStatement.toString();
        }
        catch(Exception e)
        {
            throw new RuntimeException(e);
        }
    }    
    //--------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * 
     * @param username
     * @return
     */
    private SAMLSubject getSubject(String username) throws SAMLException
    {
       SAMLSubject subject = null;
       
       SAMLNameIdentifier id = new SAMLNameIdentifier();
       id.setName(username);
       id.setFormat(SAMLNameIdentifier.FORMAT_UNSPECIFIED);                
       subject = new SAMLSubject();
       subject.setNameIdentifier(id);
       
       return subject;
    }
}
