/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.idm;

import java.util.Properties;
import java.util.Map;
import java.util.HashMap;
import java.io.InputStream;
import java.lang.reflect.Constructor;

//jmx-service related
import javax.management.MBeanServer;
import javax.management.ObjectName;
import org.jboss.mx.util.MBeanProxy;
import org.jboss.mx.util.MBeanServerLocator;

import org.apache.log4j.Logger;

//xml classes
import org.w3c.dom.*;

//jboss-security-common library
import org.jboss.security.util.Tools;

/**
 * IdentityManager provides an implementation of the IdentityManagerService that can be managed via a jmx-console.
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public class IdentityManager implements IdentityManagerService
{
    private static final Logger log = Logger.getLogger(IdentityManager.class);
    
    /**
     * 
     */
	private Map loginProviders = new HashMap();
    private Map provisioningProviders = new HashMap();
    private String confLocation = null;
    
    
    //xml constants
    private static final String identity_management = "identity-management";    
    private static final String classAttr = "class";
    private static final String id = "id";
    private static final String property = "property";
    private static final String name = "name";
    private static final String value = "value";
    private static final String provisioning = "provisioning";
    private static final String login = "login";
    private static final String provider = "provider";
    
    	
    /**
     * Initializes the IdentityManager
     *
     */
	public IdentityManager()
	{
	}                        
    //--------------------------------------------------------------------------------------------------------------------------------------------------
    /**
     * Sets the location where the configuration for this service can be looked up
     * 
     * @param confLocation
     */
    public void setConf(String confLocation)
    {
        this.confLocation = confLocation;
    }
    
    /**
     * Returns the LoginProvider corresponding to its registration id
     * 
     * @param id a unique id to identify the LoginProvider being registered. Must start with si:
     * @return
     */
    public LoginProvider getLoginProvider(String id) throws IdentityException
    {
        LoginProvider provider = null;
        
        provider = (LoginProvider)this.loginProviders.get(id);
        
        if(provider==null)
        {
            throw new IdentityException("LoginProvider not found - "+id);
        }
        return provider;
    }
    
    /**
     * Returns the default LoginProvider registered with the service
     * 
     * @return
     * @throws IdentityException
     */
    public LoginProvider getLoginProvider() throws IdentityException
    {
        LoginProvider provider = null;
        
        if(this.loginProviders!=null && this.loginProviders.size()>0)
        {
            provider = (LoginProvider)this.loginProviders.values().iterator().next();
        }
        
        if(provider==null)
        {
            throw new IdentityException("There are no LoginProviders registered with the system");
        }
        
        return provider;
    }
    
    /**
     * Returns the ProvisioningProvider corresponding to its registration id
     * 
     * @param id a unique id to identify the ProvisioningProvider being registered. Must start with si:
     * @return
     */
    public ProvisioningProvider getProvisioningProvider(String id) throws IdentityException
    {
        ProvisioningProvider provider = null;
        
        provider = (ProvisioningProvider)this.provisioningProviders.get(id);
        
        if(provider==null)
        {
            throw new IdentityException("ProvisioningProvider not found - "+id);
        }
        return provider;
    }
    
    /**
     * Returns the default ProvisioningProvider registered with the service
     * 
     * @return
     * @throws IdentityException
     */
    public ProvisioningProvider getProvisioningProvider() throws IdentityException
    {
        ProvisioningProvider provider = null;
        
        if(this.provisioningProviders!=null && this.provisioningProviders.size()>0)
        {
            provider = (ProvisioningProvider)this.provisioningProviders.values().iterator().next();
        }
        
        if(provider==null)
        {
            throw new IdentityException("There are no ProvisioningProviders registered with the system");
        }
        
        return provider;
    }
    
    /**
     * Starts the JMX service
     *
     */
    public void start()
    { 
        this.loginProviders = new HashMap();
        this.provisioningProviders = new HashMap();
        
        InputStream is = null;
        boolean success = false;
        try
        {
            if(!this.confLocation.startsWith("/"))
            {
                this.confLocation = "/" + this.confLocation;
            }
            
            is = IdentityManager.class.getResourceAsStream(this.confLocation);
                        
            //load the configuration
            Document configDocument = Tools.parseXml(is);
            
            //process the xml
            this.loadConfiguration(configDocument);
            
            log.info("Configuration successfully loaded for the IdentityManager...........");
            
            success = true;
        }
        catch(Exception e)
        {
            log.error(IdentityManager.class.getName(),e);
            log.info("Configuration loading for the IdentityManager failed...........");
            this.stop();
        }
        finally
        {
            if(is!=null)
            {
                try{is.close();}catch(Exception e){}
            }            
        }
    }
    
    /**
     * Stops the JMX service
     *
     */
    public void stop()
    {
       this.confLocation = null;
       this.loginProviders = null;
       this.provisioningProviders = null;
    }
    
    /**
     * 
     * @param document
     * @throws Exception
     */
    private void loadConfiguration(Document document) throws Exception
    {
        NodeList identityManagementNodes = document.getDocumentElement().getElementsByTagName(IdentityManager.identity_management);
        
        if(identityManagementNodes == null || identityManagementNodes.getLength()==0)
        {
            throw new IllegalArgumentException(IdentityManager.identity_management+" is missing....Please check your configuration");
        }
        
        Element identityManagement = (Element)identityManagementNodes.item(0);
        
        //login provider configuration
        Element login = (Element)identityManagement.getElementsByTagName(IdentityManager.login).item(0);
        if(login == null)
        {
            throw new IllegalArgumentException(IdentityManager.login+" is missing....Please check your configuration");
        }
        
        //provisioning provider configuration...this is optional configuration for now until
        //Federated Provisioning is integrated into the architecture
        Element provisioning = (Element)identityManagement.getElementsByTagName(IdentityManager.provisioning).item(0);
        
                
        this.loadLoginProviders(login);
        
        if(provisioning != null)
        {
            this.loadProvisioningProviders(provisioning);
        }
    }
    
    /**
     * 
     * @param login
     */
    private void loadLoginProviders(Element login) throws Exception
    {
        NodeList providerNodes = login.getElementsByTagName(IdentityManager.provider);
        for(int i=0;i<providerNodes.getLength();i++)
        {
            Element providerElement = (Element)providerNodes.item(i);
            String id = providerElement.getAttribute(IdentityManager.id);
            String providerClass = providerElement.getAttribute(IdentityManager.classAttr);
            
            //parse the properties associated with the provider
            Properties properties = new Properties();
            NodeList propertyNodes = providerElement.getElementsByTagName(IdentityManager.property);
            for(int j=0;j<propertyNodes.getLength();j++)
            {
                Element property = (Element)propertyNodes.item(j);
                String name = property.getAttribute(IdentityManager.name).trim();
                String value = "";
                if(property.getFirstChild()!=null)
                {
                    value = property.getFirstChild().getNodeValue().trim();
                }
                properties.setProperty(name,value);
            }
            
            //instantiate the identity provider
            LoginProvider provider = (LoginProvider)Class.forName(providerClass).newInstance();
            provider.setId(id);
            provider.setProperties(properties);
            this.addLoginProvider(provider);
        }
    }
    
    /**
     * 
     * @param login
     */
    private void loadProvisioningProviders(Element provisioning) throws Exception
    {
        NodeList providerNodes = provisioning.getElementsByTagName(IdentityManager.provider);
        for(int i=0;i<providerNodes.getLength();i++)
        {
            Element providerElement = (Element)providerNodes.item(i);
            String id = providerElement.getAttribute(IdentityManager.id);
            String providerClass = providerElement.getAttribute(IdentityManager.classAttr);
            
            //parse the properties associated with the provider
            Properties properties = new Properties();
            NodeList propertyNodes = providerElement.getElementsByTagName(IdentityManager.property);
            for(int j=0;j<propertyNodes.getLength();j++)
            {
                Element property = (Element)propertyNodes.item(j);
                String name = property.getAttribute(IdentityManager.name).trim();
                String value = "";
                if(property.getFirstChild()!=null)
                {
                    value = property.getFirstChild().getNodeValue().trim();
                }
                properties.setProperty(name,value);
            }
            
            //instantiate the identity provider
            ProvisioningProvider provider = (ProvisioningProvider)Class.forName(providerClass).newInstance();
            provider.setId(id);
            provider.setProperties(properties);
            this.addProvisioningProvider(provider);
        }
    }
    
        
    /**
     * 
     * @param id
     * @return
     */
    private boolean validateProviderId(String id)
    {
        boolean isValid = false;
        
        if(id.indexOf(":")!=-1)
        {
            String firstToken = id.substring(0,id.indexOf(':'));
            if(firstToken.equals("si"))
            {
                isValid = true;
            }
        }
        
        return isValid;
    }
                	    
    /**
     * 
     * @param provider
     */
    private void addLoginProvider(LoginProvider provider)
    {
        String id = provider.getId();
        
        if(id!=null && id.trim().length()>0)
        {
            if(this.loginProviders.containsKey(id))
            {
                Object cour = this.loginProviders.get(id);
                String className = cour.getClass().getName();
                if(provider.getClass().getName().trim().equals(className.trim()))
                {
                    throw new IdentityException("A LoginProvider with this Id is already registered. Try to make your LoginProviderId unique");
                }
                else
                {
                    //an identityprovider with this id is already registered, don't bother registering a new implementation
                    return;
                }
            } 
            
            if(this.validateProviderId(provider.getId()))
            {
                //go ahead and register this IdentityProvider with this IdentityManager
                this.loginProviders.put(provider.getId(),provider);
            }
            else
            {
                throw new IdentityException("Invalid LoginProvider Id. Make sure the id is of the form ssi:<vendor name>:<vendor specific information>");
            }
        }
        else
        {
            throw new IdentityException("Invalid LoginProvider Id. Null Id not allowed");
        }
    }
    
    /**
     * 
     * @param provider
     */
    private void addProvisioningProvider(ProvisioningProvider provider)
    {
        String id = provider.getId();
        
        if(id!=null && id.trim().length()>0)
        {
            if(this.provisioningProviders.containsKey(id))
            {
                Object cour = this.provisioningProviders.get(id);
                String className = cour.getClass().getName();
                if(provider.getClass().getName().trim().equals(className.trim()))
                {
                    throw new IdentityException("A ProvisioningProvider with this Id is already registered. Try to make your ProvisioningProviderId unique");
                }
                else
                {
                    //an identityprovider with this id is already registered, don't bother registering a new implementation
                    return;
                }
            } 
            
            if(this.validateProviderId(provider.getId()))
            {
                //go ahead and register this IdentityProvider with this IdentityManager
                this.provisioningProviders.put(provider.getId(),provider);
            }
            else
            {
                throw new IdentityException("Invalid ProvisioningProvider Id. Make sure the id is of the form ssi:<vendor name>:<vendor specific information>");
            }
        }
        else
        {
            throw new IdentityException("Invalid ProvisioningProvider Id. Null Id not allowed");
        }
    }	
    //-------utility related methods-------------------------------------------------------------------------------
    /**
     * Finds and returns the LoginProvider registered with the specified unique id
     * 
     * @param id unique id of the LoginProvider 
     * @return the LoginProvider that matches the id
     * @throws IdentityException
     */
    public static LoginProvider findLoginProvider(String id) throws IdentityException
    { 
        try
        {
            LoginProvider provider = null;
            
            //get the registered identityManager service mbean
            MBeanServer mbeanServer = MBeanServerLocator.locateJBoss();
            IdentityManagerService identityManager = (IdentityManagerService)MBeanProxy.get
            (IdentityManagerService.class,
            new ObjectName("jboss.sso:service=IdentityManager"), 
            mbeanServer);
            
            provider = identityManager.getLoginProvider(id);
                    
            return provider;
        }
        catch(Exception e)
        {
            throw new IdentityException(e);
        }
    }
    
    /**
     * Finds and returns the default LoginProvider registered with the service
     * 
     * @return default LoginProvider
     * @throws IdentityException
     */
    public static LoginProvider findLoginProvider() throws IdentityException
    { 
        try
        {
            LoginProvider provider = null;
            
            //get the registered identityManager service mbean
            MBeanServer mbeanServer = MBeanServerLocator.locateJBoss();
            IdentityManagerService identityManager = (IdentityManagerService)MBeanProxy.get
            (IdentityManagerService.class,
            new ObjectName("jboss.sso:service=IdentityManager"), 
            mbeanServer);
            
            provider = identityManager.getLoginProvider();
                    
            return provider;
        }
        catch(Exception e)
        {
            throw new IdentityException(e);
        }
    }
    
    /**
     * Finds and returns the ProvisioningProvider registered with the specified unique id
     * 
     * @param id unique id
     * @return the ProvisioningProvider corresponding to the specified id
     * @throws IdentityException
     */
    public static ProvisioningProvider findProvisioningProvider(String id) throws IdentityException
    { 
        try
        {
            ProvisioningProvider provider = null;
            
            //get the registered identityManager service mbean
            MBeanServer mbeanServer = MBeanServerLocator.locateJBoss();
            IdentityManagerService identityManager = (IdentityManagerService)MBeanProxy.get
            (IdentityManagerService.class,
            new ObjectName("jboss.sso:service=IdentityManager"), 
            mbeanServer);
            
            provider = identityManager.getProvisioningProvider(id);
                    
            return provider;
        }
        catch(Exception e)
        {
            throw new IdentityException(e);
        }
    }
    
    /**
     * Finds and returns the default ProvisioningProvider registered with the service
     * 
     * @return the default ProvisioningProvider
     * @throws IdentityException
     */
    public static ProvisioningProvider findProvisioningProvider() throws IdentityException
    { 
        try
        {
            ProvisioningProvider provider = null;
            
            //get the registered identityManager service mbean
            MBeanServer mbeanServer = MBeanServerLocator.locateJBoss();
            IdentityManagerService identityManager = (IdentityManagerService)MBeanProxy.get
            (IdentityManagerService.class,
            new ObjectName("jboss.sso:service=IdentityManager"), 
            mbeanServer);
            
            provider = identityManager.getProvisioningProvider();
                    
            return provider;
        }
        catch(Exception e)
        {
            throw new IdentityException(e);
        }
    }
}
