/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.idm;

import java.security.Principal;
import java.util.Collection;
import java.util.Properties;

/**
 * The interface <b>org.jboss.security.idm.LoginProvider</b> represents a data source component that extracts data from Identity Stores like
 * LDAP repositories, ActiveX Directories, thirdparty identity management systems like SiteMinder, or even custom JDBC based systems.<br/><br/>
 * 
 * It serves as an abstraction to Identity data related to authentication on the system.
 * 
 * @author Sohil Shah - sohil.shah@jboss.com - Sep 20, 2006
 *
 */
public interface LoginProvider 
{
    /**
     * Returns an id that uniquely identifies the LoginProvider plugged into the JBoss SSO system. 
     * It should be of the form: si:<a unique string such as vendor name, etc. eg. jboss>:<optional parameter>
     * 
     * @return  id of the LoginProvider
     * @throws IdentityException
     */
    public String getId() throws IdentityException;
    
    /**
     * Sets the unique id for the LoginProvider
     * 
     * @param id
     * @throws IdentityException
     */
    public void setId(String id) throws IdentityException;
    
    /**
     * Sets any properties like LDAP URLs, JDBC configuration, etc needed to configure this particular implementation
     * of the LoginProvider
     * 
     * @param properties
     * @throws IdentityException
     */
    public void setProperties(Properties properties) throws IdentityException;
    
    /**
     * Returns the Identity data for the specified Principal
     * 
     * @param principal
     * @return Identity data for the Principal
     * @throws IdentityException
     */
    public Identity read(Principal principal) throws IdentityException;
    
    /**
     * Returns the Identity data for the specified username
     * 
     * @param username
     * @return Identity data for the username
     * @throws IdentityException
     */
    public Identity read(String username) throws IdentityException;
    
    /**
     * Checks if the specified Principal exists in the system anymore or not
     * 
     * @param principal
     * @return
     * @throws IdentityException
     */
    public boolean exists(Principal principal) throws IdentityException;
    
    /**
     * Checks if the specified username exists in the system anymore or not
     * 
     * @param username
     * @return
     * @throws IdentityException
     */
    public boolean exists(String username) throws IdentityException;
    
    /**
     * Checks if the specified Principal should successfully authenticated using the specified password
     * 
     * @param principal
     * @param password
     * @return
     * @throws IdentityException
     */
    public boolean login(Principal principal,byte[] password) throws IdentityException;
    
    /**
     * Checks if the specified username should be successfully authenticated using the specified password
     * 
     * @param username
     * @param password
     * @return
     * @throws IdentityException
     */
    public boolean login(String username,byte[] password) throws IdentityException;
    
    /**
     * Return all the authorization roles registered in the system. Note: this is a list of all the roles, and not
     * specific to any particular user
     * 
     * @return a Collection of all the authorization roles registered in the system
     * @throws IdentityException
     */
    public Collection readAllRoles() throws IdentityException;
    
    /**
     * This notification is received from the SSO Engine when the engine performs an automatic logic based 
     * on a trusted assertion received from a partner site. The method can be used by the application to setup its
     * environment in response to the user logged in to the system
     * 
     * @param loginContext
     * @throws IdentityException
     */
    public void processSSOLoginNotification(LoginContext loginContext) throws IdentityException;
        
}
